﻿/* EntitiesToDTOs. Copyright (c) 2011. Fabian Fernandez.
 * http://entitiestodtos.codeplex.com
 * Licensed by Common Development and Distribution License (CDDL).
 * http://entitiestodtos.codeplex.com/license
 * Fabian Fernandez. 
 * http://www.linkedin.com/in/fabianfernandezb/en
 * */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using EntitiesToDTOs.Properties;
using EntitiesToDTOs.Enums;
using EntitiesToDTOs.Helpers;

namespace EntitiesToDTOs.Domain
{
    /// <summary>
    /// Represents an Entity Association.
    /// </summary>
    internal class EntityAssociation
    {
        /// <summary>
        /// Association Name.
        /// </summary>
        public string AssociationName { get; private set; }

        /// <summary>
        /// DTO Name.
        /// </summary>
        public string DTOName { get; private set; }

        /// <summary>
        /// Association Role Name.
        /// </summary>
        public string EndRoleName { get; private set; }

        /// <summary>
        /// End Multiplicity Type.
        /// </summary>
        public EntityAssociationMultiplicityEnum EndMultiplicity { get; private set; }


        public EntityAssociation(XElement associationNode, EntityAssociationEndEnum entityAssociationEndDesired)
        {
            // Set Association Name
            this.AssociationName = associationNode.Attribute(EdmxNodeAttributes.Association_Name).Value;

            // Get End nodes
            XElement[] endNodes = associationNode.Descendants(EdmxNodes.End).ToArray();

            // Get the position of the desired End node
            int endPosition = (entityAssociationEndDesired == EntityAssociationEndEnum.First ? 0 : 1);

            // Set DTO Name
            this.DTOName = endNodes[endPosition].Attribute(EdmxNodeAttributes.End_Type).Value;
            string[] entityNameSplitted = this.DTOName.Split(new string[] { Resources.Dot }, StringSplitOptions.RemoveEmptyEntries);
            this.DTOName = entityNameSplitted[(entityNameSplitted.Length - 1)];
            this.DTOName = EntityDTOHelper.GetDTOName(this.DTOName);

            // Set End Role Name
            this.EndRoleName = endNodes[endPosition].Attribute(EdmxNodeAttributes.End_Role).Value;

            // Set Multiplicity
            this.EndMultiplicity = EntityAssociationHelper.GetMultiplicity(endNodes[endPosition]);
        }
    }
}